@@ -1,7 +1,7 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: October 20, 2018
+# Last Change: March 1, 2020
 # URL: https://humanfriendly.readthedocs.io
 
 """The main module of the `humanfriendly` package."""
@@ -22,25 +22,78 @@ import time
 # using characters like dashes and vertical bars to emulate borders. Since then
 # support for other tables has been added and the name of format_table() has
 # changed. The following import statement preserves backwards compatibility.
-from humanfriendly.tables import format_pretty_table as format_table  # NOQA
+from humanfriendly.tables import format_pretty_table as format_table
 
 # In humanfriendly 1.30 the following text manipulation functions were moved
 # out into a separate module to enable their usage in other modules of the
 # humanfriendly package (without causing circular imports).
-from humanfriendly.text import (  # NOQA
-    compact, concatenate, dedent, format, is_empty_line,
-    pluralize, tokenize, trim_empty_lines,
+from humanfriendly.text import (
+    compact,
+    concatenate,
+    dedent,
+    format,
+    is_empty_line,
+    pluralize,
+    tokenize,
+    trim_empty_lines,
 )
 
 # In humanfriendly 1.38 the prompt_for_choice() function was moved out into a
 # separate module because several variants of interactive prompts were added.
-from humanfriendly.prompts import prompt_for_choice  # NOQA
+from humanfriendly.prompts import prompt_for_choice
 
 # Compatibility with Python 2 and 3.
 from humanfriendly.compat import is_string, monotonic
 
+# Public identifiers that require documentation.
+__all__ = (
+    # Variables imported from other modules.
+    'compact',
+    'concatenate',
+    'dedent',
+    'format',
+    'format_table',
+    'is_empty_line',
+    'pluralize',
+    'prompt_for_choice',
+    'tokenize',
+    'trim_empty_lines',
+    # Variables defined in this module.
+    'AutomaticSpinner',
+    'CombinedUnit',
+    'InvalidDate',
+    'InvalidLength',
+    'InvalidSize',
+    'InvalidTimespan',
+    'SizeUnit',
+    'Spinner',
+    'Timer',
+    '__version__',
+    'coerce_boolean',
+    'coerce_pattern',
+    'coerce_seconds',
+    'disk_size_units',
+    'erase_line_code',
+    'format_length',
+    'format_number',
+    'format_path',
+    'format_size',
+    'format_timespan',
+    'hide_cursor_code',
+    'length_size_units',
+    'minimum_spinner_interval',
+    'parse_date',
+    'parse_length',
+    'parse_path',
+    'parse_size',
+    'parse_timespan',
+    'round_number',
+    'show_cursor_code',
+    'time_units',
+)
+
 # Semi-standard module versioning.
-__version__ = '4.17'
+__version__ = '7.2'
 
 # Spinners are redrawn at most this many seconds.
 minimum_spinner_interval = 0.2
@@ -77,7 +130,9 @@ length_size_units = (dict(prefix='nm', divider=1e-09, singular='nm', plural='nm'
                      dict(prefix='km', divider=1000, singular='km', plural='km'))
 
 # Common time units, used for formatting of time spans.
-time_units = (dict(divider=1e-3, singular='millisecond', plural='milliseconds', abbreviations=['ms']),
+time_units = (dict(divider=1e-9, singular='nanosecond', plural='nanoseconds', abbreviations=['ns']),
+              dict(divider=1e-6, singular='microsecond', plural='microseconds', abbreviations=['us']),
+              dict(divider=1e-3, singular='millisecond', plural='milliseconds', abbreviations=['ms']),
               dict(divider=1, singular='second', plural='seconds', abbreviations=['s', 'sec', 'secs']),
               dict(divider=60, singular='minute', plural='minutes', abbreviations=['m', 'min', 'mins']),
               dict(divider=60 * 60, singular='hour', plural='hours', abbreviations=['h']),
@@ -96,7 +151,7 @@ def coerce_boolean(value):
                   - The strings '0', 'no', 'false' and 'off' are coerced to :data:`False`.
                   - Other strings raise an exception.
 
-                  Other Python values are coerced using :func:`bool()`.
+                  Other Python values are coerced using :class:`bool`.
     :returns: A proper boolean value.
     :raises: :exc:`exceptions.ValueError` when the value is a string but
              cannot be coerced with certainty.
@@ -145,14 +200,10 @@ def coerce_seconds(value):
     :returns: An :class:`int` or :class:`float` value.
 
     When `value` is a :class:`datetime.timedelta` object the
-    :func:`~datetime.timedelta.total_seconds()` method is called.
-    On Python 2.6 this method is not available so it is emulated.
+    :meth:`~datetime.timedelta.total_seconds()` method is called.
     """
     if isinstance(value, datetime.timedelta):
-        if hasattr(value, 'total_seconds'):
-            return value.total_seconds()
-        else:
-            return (value.microseconds + (value.seconds + value.days * 24 * 3600) * 10**6) / 10**6
+        return value.total_seconds()
     if not isinstance(value, numbers.Number):
         msg = "Failed to coerce value to number of seconds! (%r)"
         raise ValueError(format(msg, value))
@@ -241,6 +292,9 @@ def parse_size(size, binary=False):
             return int(tokens[0])
         # Otherwise we expect two tokens: A number and a unit.
         if normalized_unit:
+            # Convert plural units to singular units, for details:
+            # https://github.com/xolox/python-humanfriendly/issues/26
+            normalized_unit = normalized_unit.rstrip('s')
             for unit in disk_size_units:
                 # First we check for unambiguous symbols (KiB, MiB, GiB, etc)
                 # and names (kibibyte, mebibyte, gibibyte, etc) because their
@@ -314,7 +368,7 @@ def parse_length(length):
     if tokens and isinstance(tokens[0], numbers.Number):
         # If the input contains only a number, it's assumed to be the number of metres.
         if len(tokens) == 1:
-            return int(tokens[0])
+            return tokens[0]
         # Otherwise we expect to find two tokens: A number and a unit.
         if len(tokens) == 2 and is_string(tokens[1]):
             normalized_unit = tokens[1].lower()
@@ -391,7 +445,7 @@ def round_number(count, keep_width=False):
     text = '%.2f' % float(count)
     if not keep_width:
         text = re.sub('0+$', '', text)
-        text = re.sub('\.$', '', text)
+        text = re.sub(r'\.$', '', text)
     return text
 
 
@@ -432,7 +486,7 @@ def format_timespan(num_seconds, detailed=False, max_units=3):
         # Slow path.
         result = []
         num_seconds = decimal.Decimal(str(num_seconds))
-        relevant_units = list(reversed(time_units[0 if detailed else 1:]))
+        relevant_units = list(reversed(time_units[0 if detailed else 3:]))
         for unit in relevant_units:
             # Extract the unit count from the remaining time.
             divider = decimal.Decimal(str(unit['divider']))
