@@ -1,17 +1,22 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: July 21, 2018
+# Last Change: March 1, 2020
 # URL: https://humanfriendly.readthedocs.io
 
 """
-Interaction with UNIX terminals.
+Interaction with interactive text terminals.
 
-The :mod:`~humanfriendly.terminal` module makes it easy to interact with UNIX
-terminals and format text for rendering on UNIX terminals. If the terms used in
-the documentation of this module don't make sense to you then please refer to
-the `Wikipedia article on ANSI escape sequences`_ for details about how ANSI
-escape sequences work.
+The :mod:`~humanfriendly.terminal` module makes it easy to interact with
+interactive text terminals and format text for rendering on such terminals. If
+the terms used in the documentation of this module don't make sense to you then
+please refer to the `Wikipedia article on ANSI escape sequences`_ for details
+about how ANSI escape sequences work.
+
+This module was originally developed for use on UNIX systems, but since then
+Windows 10 gained native support for ANSI escape sequences and this module was
+enhanced to recognize and support this. For details please refer to the
+:func:`enable_ansi_support()` function.
 
 .. _Wikipedia article on ANSI escape sequences: http://en.wikipedia.org/wiki/ANSI_escape_code#Sequence_elements
 """
@@ -20,6 +25,7 @@ escape sequences work.
 import codecs
 import numbers
 import os
+import platform
 import re
 import subprocess
 import sys
@@ -38,10 +44,58 @@ except ImportError:
 # Modules included in our package. We import find_meta_variables() here to
 # preserve backwards compatibility with older versions of humanfriendly where
 # that function was defined in this module.
-from humanfriendly.compat import HTMLParser, StringIO, coerce_string, name2codepoint, is_unicode, unichr
+from humanfriendly.compat import (
+    HTMLParser,
+    StringIO,
+    coerce_string,
+    is_unicode,
+    name2codepoint,
+    on_windows,
+    unichr,
+    which,
+)
+from humanfriendly.decorators import cached
 from humanfriendly.text import compact_empty_lines, concatenate, format
 from humanfriendly.usage import find_meta_variables, format_usage  # NOQA
 
+# Public identifiers that require documentation.
+__all__ = (
+    'ANSI_COLOR_CODES',
+    'ANSI_CSI',
+    'ANSI_ERASE_LINE',
+    'ANSI_RESET',
+    'ANSI_SGR',
+    'ANSI_TEXT_STYLES',
+    'CLEAN_OUTPUT_PATTERN',
+    'DEFAULT_COLUMNS',
+    'DEFAULT_ENCODING',
+    'DEFAULT_LINES',
+    'HIGHLIGHT_COLOR',
+    'HTMLConverter',
+    'ansi_strip',
+    'ansi_style',
+    'ansi_width',
+    'ansi_wrap',
+    'auto_encode',
+    'clean_terminal_output',
+    'connected_to_terminal',
+    'enable_ansi_support',
+    'find_terminal_size',
+    'find_terminal_size_using_ioctl',
+    'find_terminal_size_using_stty',
+    'get_pager_command',
+    'have_windows_native_ansi_support',
+    'html_to_ansi',
+    'message',
+    'output',
+    'readline_strip',
+    'readline_wrap',
+    'show_pager',
+    'terminal_supports_colors',
+    'usage',
+    'warning',
+)
+
 ANSI_CSI = '\x1b['
 """The ANSI "Control Sequence Introducer" (a string)."""
 
@@ -96,80 +150,6 @@ message of the ``humanfriendly`` program). If the environment variable
 """
 
 
-def output(text, *args, **kw):
-    """
-    Print a formatted message to the standard output stream.
-
-    For details about argument handling please refer to
-    :func:`~humanfriendly.text.format()`.
-
-    Renders the message using :func:`~humanfriendly.text.format()` and writes
-    the resulting string (followed by a newline) to :data:`sys.stdout` using
-    :func:`auto_encode()`.
-    """
-    auto_encode(sys.stdout, coerce_string(text) + '\n', *args, **kw)
-
-
-def message(text, *args, **kw):
-    """
-    Print a formatted message to the standard error stream.
-
-    For details about argument handling please refer to
-    :func:`~humanfriendly.text.format()`.
-
-    Renders the message using :func:`~humanfriendly.text.format()` and writes
-    the resulting string (followed by a newline) to :data:`sys.stderr` using
-    :func:`auto_encode()`.
-    """
-    auto_encode(sys.stderr, coerce_string(text) + '\n', *args, **kw)
-
-
-def warning(text, *args, **kw):
-    """
-    Show a warning message on the terminal.
-
-    For details about argument handling please refer to
-    :func:`~humanfriendly.text.format()`.
-
-    Renders the message using :func:`~humanfriendly.text.format()` and writes
-    the resulting string (followed by a newline) to :data:`sys.stderr` using
-    :func:`auto_encode()`.
-
-    If :data:`sys.stderr` is connected to a terminal that supports colors,
-    :func:`ansi_wrap()` is used to color the message in a red font (to make
-    the warning stand out from surrounding text).
-    """
-    text = coerce_string(text)
-    if terminal_supports_colors(sys.stderr):
-        text = ansi_wrap(text, color='red')
-    auto_encode(sys.stderr, text + '\n', *args, **kw)
-
-
-def auto_encode(stream, text, *args, **kw):
-    """
-    Reliably write Unicode strings to the terminal.
-
-    :param stream: The file-like object to write to (a value like
-                   :data:`sys.stdout` or :data:`sys.stderr`).
-    :param text: The text to write to the stream (a string).
-    :param args: Refer to :func:`~humanfriendly.text.format()`.
-    :param kw: Refer to :func:`~humanfriendly.text.format()`.
-
-    Renders the text using :func:`~humanfriendly.text.format()` and writes it
-    to the given stream. If an :exc:`~exceptions.UnicodeEncodeError` is
-    encountered in doing so, the text is encoded using :data:`DEFAULT_ENCODING`
-    and the write is retried. The reasoning behind this rather blunt approach
-    is that it's preferable to get output on the command line in the wrong
-    encoding then to have the Python program blow up with a
-    :exc:`~exceptions.UnicodeEncodeError` exception.
-    """
-    text = format(text, *args, **kw)
-    try:
-        stream.write(text)
-    except UnicodeEncodeError:
-        stream.write(codecs.encode(text, DEFAULT_ENCODING))
-
-
 def ansi_strip(text, readline_hints=True):
     """
     Strip ANSI escape sequences from the given string.
@@ -324,26 +304,29 @@ def ansi_wrap(text, **kw):
         return text
 
 
-def readline_wrap(expr):
+def auto_encode(stream, text, *args, **kw):
     """
-    Wrap an ANSI escape sequence in `readline hints`_.
+    Reliably write Unicode strings to the terminal.
 
-    :param text: The text with the escape sequence to wrap (a string).
-    :returns: The wrapped text.
+    :param stream: The file-like object to write to (a value like
+                   :data:`sys.stdout` or :data:`sys.stderr`).
+    :param text: The text to write to the stream (a string).
+    :param args: Refer to :func:`~humanfriendly.text.format()`.
+    :param kw: Refer to :func:`~humanfriendly.text.format()`.
 
-    .. _readline hints: http://superuser.com/a/301355
+    Renders the text using :func:`~humanfriendly.text.format()` and writes it
+    to the given stream. If an :exc:`~exceptions.UnicodeEncodeError` is
+    encountered in doing so, the text is encoded using :data:`DEFAULT_ENCODING`
+    and the write is retried. The reasoning behind this rather blunt approach
+    is that it's preferable to get output on the command line in the wrong
+    encoding then to have the Python program blow up with a
+    :exc:`~exceptions.UnicodeEncodeError` exception.
     """
-    return '\001' + expr + '\002'
-
-
-def readline_strip(expr):
-    """
-    Remove `readline hints`_ from a string.
-
-    :param text: The text to strip (a string).
-    :returns: The stripped text.
-    """
-    return expr.replace('\001', '').replace('\002', '')
+    text = format(text, *args, **kw)
+    try:
+        stream.write(text)
+    except UnicodeEncodeError:
+        stream.write(codecs.encode(text, DEFAULT_ENCODING))
 
 
 def clean_terminal_output(text):
@@ -357,13 +340,13 @@ def clean_terminal_output(text):
     (0x0D) and line feed (0x0A) characters and the ANSI 'erase line' escape
     sequence on interactive terminals. It's intended to clean up command output
     that was originally meant to be rendered on an interactive terminal and
-    that has been captured using e.g. the script_ program [#]_ or the
+    that has been captured using e.g. the :man:`script` program [#]_ or the
     :mod:`pty` module [#]_.
 
     .. [#] My coloredlogs_ package supports the ``coloredlogs --to-html``
-           command which uses script_ to fool a subprocess into thinking that
-           it's connected to an interactive terminal (in order to get it to
-           emit ANSI escape sequences).
+           command which uses :man:`script` to fool a subprocess into thinking
+           that it's connected to an interactive terminal (in order to get it
+           to emit ANSI escape sequences).
 
     .. [#] My capturer_ package uses the :mod:`pty` module to fool the current
            process and subprocesses into thinking they are connected to an
@@ -384,9 +367,8 @@ def clean_terminal_output(text):
       contain only the closing end of the ANSI escape sequence pair. Tracking
       this kind of complexity requires a state machine and proper parsing.
 
-    .. _capturer: https://pypi.python.org/pypi/capturer
-    .. _coloredlogs: https://pypi.python.org/pypi/coloredlogs
-    .. _script: http://man7.org/linux/man-pages/man1/script.1.html
+    .. _capturer: https://pypi.org/project/capturer
+    .. _coloredlogs: https://pypi.org/project/coloredlogs
     """
     cleaned_lines = []
     current_line = ''
@@ -439,38 +421,56 @@ def connected_to_terminal(stream=None):
         return False
 
 
-def html_to_ansi(data, callback=None):
+@cached
+def enable_ansi_support():
     """
-    Convert HTML with simple text formatting to text with ANSI escape sequences.
+    Try to enable support for ANSI escape sequences (required on Windows).
 
-    :param data: The HTML to convert (a string).
-    :param callback: Optional callback to pass to :class:`HTMLConverter`.
-    :returns: Text with ANSI escape sequences (a string).
+    :returns: :data:`True` if ANSI is supported, :data:`False` otherwise.
 
-    Please refer to the documentation of the :class:`HTMLConverter` class for
-    details about the conversion process (like which tags are supported) and an
-    example with a screenshot.
+    This functions checks for the following supported configurations, in the
+    given order:
+
+    1. On Windows, if :func:`have_windows_native_ansi_support()` confirms
+       native support for ANSI escape sequences :mod:`ctypes` will be used to
+       enable this support.
+
+    2. On Windows, if the environment variable ``$ANSICON`` is set nothing is
+       done because it is assumed that support for ANSI escape sequences has
+       already been enabled via `ansicon <https://github.com/adoxa/ansicon>`_.
+
+    3. On Windows, an attempt is made to import and initialize the Python
+       package :pypi:`colorama` instead (of course for this to work
+       :pypi:`colorama` has to be installed).
+
+    4. On other platforms this function calls :func:`connected_to_terminal()`
+       to determine whether ANSI escape sequences are supported (that is to
+       say all platforms that are not Windows are assumed to support ANSI
+       escape sequences natively, without weird contortions like above).
+
+       This makes it possible to call :func:`enable_ansi_support()`
+       unconditionally without checking the current platform.
+
+    The :func:`~humanfriendly.decorators.cached` decorator is used to ensure
+    that this function is only executed once, but its return value remains
+    available on later calls.
     """
-    converter = HTMLConverter(callback=callback)
-    return converter(data)
-
-
-def terminal_supports_colors(stream=None):
-    """
-    Check if a stream is connected to a terminal that supports ANSI escape sequences.
-
-    :param stream: The stream to check (a file-like object,
-                   defaults to :data:`sys.stdout`).
-    :returns: :data:`True` if the terminal supports ANSI escape sequences,
-              :data:`False` otherwise.
-
-    This function is inspired by the implementation of
-    `django.core.management.color.supports_color()
-    <https://github.com/django/django/blob/master/django/core/management/color.py>`_.
-    """
-    return (sys.platform != 'Pocket PC' and
-            (sys.platform != 'win32' or 'ANSICON' in os.environ) and
-            connected_to_terminal(stream))
+    if have_windows_native_ansi_support():
+        import ctypes
+        ctypes.windll.kernel32.SetConsoleMode(ctypes.windll.kernel32.GetStdHandle(-11), 7)
+        ctypes.windll.kernel32.SetConsoleMode(ctypes.windll.kernel32.GetStdHandle(-12), 7)
+        return True
+    elif on_windows():
+        if 'ANSICON' in os.environ:
+            return True
+        try:
+            import colorama
+            colorama.init()
+            return True
+        except ImportError:
+            return False
+    else:
+        return connected_to_terminal()
 
 
 def find_terminal_size():
@@ -492,8 +492,8 @@ def find_terminal_size():
               the result of :func:`find_terminal_size()` should be correct.
 
               `Pre-emptive snarky comment`_: It's possible to cache the result
-              of this function and use :data:`signal.SIGWINCH` to refresh the
-              cached values!
+              of this function and use :mod:`signal.SIGWINCH <signal>` to
+              refresh the cached values!
 
               Response: As a library I don't consider it the role of the
               :mod:`humanfriendly.terminal` module to install a process wide
@@ -558,22 +558,134 @@ def find_terminal_size_using_stty():
     return tuple(map(int, tokens))
 
 
-def usage(usage_text):
+def get_pager_command(text=None):
     """
-    Print a human friendly usage message to the terminal.
+    Get the command to show a text on the terminal using a pager.
 
-    :param text: The usage message to print (a string).
+    :param text: The text to print to the terminal (a string).
+    :returns: A list of strings with the pager command and arguments.
 
-    This function does two things:
+    The use of a pager helps to avoid the wall of text effect where the user
+    has to scroll up to see where the output began (not very user friendly).
 
-    1. If :data:`sys.stdout` is connected to a terminal (see
-       :func:`connected_to_terminal()`) then the usage message is formatted
-       using :func:`.format_usage()`.
-    2. The usage message is shown using a pager (see :func:`show_pager()`).
+    If the given text contains ANSI escape sequences the command ``less
+    --RAW-CONTROL-CHARS`` is used, otherwise the environment variable
+    ``$PAGER`` is used (if ``$PAGER`` isn't set :man:`less` is used).
+
+    When the selected pager is :man:`less`, the following options are used to
+    make the experience more user friendly:
+
+    - ``--quit-if-one-screen`` causes :man:`less` to automatically exit if the
+      entire text can be displayed on the first screen. This makes the use of a
+      pager transparent for smaller texts (because the operator doesn't have to
+      quit the pager).
+
+    - ``--no-init`` prevents :man:`less` from clearing the screen when it
+      exits. This ensures that the operator gets a chance to review the text
+      (for example a usage message) after quitting the pager, while composing
+      the next command.
     """
-    if terminal_supports_colors(sys.stdout):
-        usage_text = format_usage(usage_text)
-    show_pager(usage_text)
+    # Compose the pager command.
+    if text and ANSI_CSI in text:
+        command_line = ['less', '--RAW-CONTROL-CHARS']
+    else:
+        command_line = [os.environ.get('PAGER', 'less')]
+    # Pass some additional options to `less' (to make it more
+    # user friendly) without breaking support for other pagers.
+    if os.path.basename(command_line[0]) == 'less':
+        command_line.append('--no-init')
+        command_line.append('--quit-if-one-screen')
+    return command_line
+
+
+@cached
+def have_windows_native_ansi_support():
+    """
+    Check if we're running on a Windows 10 release with native support for ANSI escape sequences.
+
+    :returns: :data:`True` if so, :data:`False` otherwise.
+
+    The :func:`~humanfriendly.decorators.cached` decorator is used as a minor
+    performance optimization. Semantically this should have zero impact because
+    the answer doesn't change in the lifetime of a computer process.
+    """
+    if on_windows():
+        try:
+            # I can't be 100% sure this will never break and I'm not in a
+            # position to test it thoroughly either, so I decided that paying
+            # the price of one additional try / except statement is worth the
+            # additional peace of mind :-).
+            components = tuple(int(c) for c in platform.version().split('.'))
+            return components >= (10, 0, 14393)
+        except Exception:
+            pass
+    return False
+
+
+def html_to_ansi(data, callback=None):
+    """
+    Convert HTML with simple text formatting to text with ANSI escape sequences.
+
+    :param data: The HTML to convert (a string).
+    :param callback: Optional callback to pass to :class:`HTMLConverter`.
+    :returns: Text with ANSI escape sequences (a string).
+
+    Please refer to the documentation of the :class:`HTMLConverter` class for
+    details about the conversion process (like which tags are supported) and an
+    example with a screenshot.
+    """
+    converter = HTMLConverter(callback=callback)
+    return converter(data)
+
+
+def message(text, *args, **kw):
+    """
+    Print a formatted message to the standard error stream.
+
+    For details about argument handling please refer to
+    :func:`~humanfriendly.text.format()`.
+
+    Renders the message using :func:`~humanfriendly.text.format()` and writes
+    the resulting string (followed by a newline) to :data:`sys.stderr` using
+    :func:`auto_encode()`.
+    """
+    auto_encode(sys.stderr, coerce_string(text) + '\n', *args, **kw)
+
+
+def output(text, *args, **kw):
+    """
+    Print a formatted message to the standard output stream.
+
+    For details about argument handling please refer to
+    :func:`~humanfriendly.text.format()`.
+
+    Renders the message using :func:`~humanfriendly.text.format()` and writes
+    the resulting string (followed by a newline) to :data:`sys.stdout` using
+    :func:`auto_encode()`.
+    """
+    auto_encode(sys.stdout, coerce_string(text) + '\n', *args, **kw)
+
+
+def readline_strip(expr):
+    """
+    Remove `readline hints`_ from a string.
+
+    :param text: The text to strip (a string).
+    :returns: The stripped text.
+    """
+    return expr.replace('\001', '').replace('\002', '')
+
+
+def readline_wrap(expr):
+    """
+    Wrap an ANSI escape sequence in `readline hints`_.
+
+    :param text: The text with the escape sequence to wrap (a string).
+    :returns: The wrapped text.
+
+    .. _readline hints: http://superuser.com/a/301355
+    """
+    return '\001' + expr + '\002'
 
 
 def show_pager(formatted_text, encoding=DEFAULT_ENCODING):
@@ -596,54 +708,78 @@ def show_pager(formatted_text, encoding=DEFAULT_ENCODING):
     that's used to invoke the pager.
     """
     if connected_to_terminal():
+        # Make sure the selected pager command is available.
         command_line = get_pager_command(formatted_text)
-        pager = subprocess.Popen(command_line, stdin=subprocess.PIPE)
-        if is_unicode(formatted_text):
-            formatted_text = formatted_text.encode(encoding)
-        pager.communicate(input=formatted_text)
-    else:
-        output(formatted_text)
+        if which(command_line[0]):
+            pager = subprocess.Popen(command_line, stdin=subprocess.PIPE)
+            if is_unicode(formatted_text):
+                formatted_text = formatted_text.encode(encoding)
+            pager.communicate(input=formatted_text)
+            return
+    output(formatted_text)
 
 
-def get_pager_command(text=None):
+def terminal_supports_colors(stream=None):
     """
-    Get the command to show a text on the terminal using a pager.
+    Check if a stream is connected to a terminal that supports ANSI escape sequences.
 
-    :param text: The text to print to the terminal (a string).
-    :returns: A list of strings with the pager command and arguments.
+    :param stream: The stream to check (a file-like object,
+                   defaults to :data:`sys.stdout`).
+    :returns: :data:`True` if the terminal supports ANSI escape sequences,
+              :data:`False` otherwise.
 
-    The use of a pager helps to avoid the wall of text effect where the user
-    has to scroll up to see where the output began (not very user friendly).
-
-    If the given text contains ANSI escape sequences the command ``less
-    --RAW-CONTROL-CHARS`` is used, otherwise the environment variable
-    ``$PAGER`` is used (if ``$PAGER`` isn't set less_ is used).
-
-    When the selected pager is less_, the following options are used to make
-    the experience more user friendly:
-
-    - ``--quit-if-one-screen`` causes less_ to automatically exit if the entire
-      text can be displayed on the first screen. This makes the use of a pager
-      transparent for smaller texts (because the operator doesn't have to quit
-      the pager).
-
-    - ``--no-init`` prevents less_ from clearing the screen when it exits. This
-      ensures that the operator gets a chance to review the text (for example a
-      usage message) after quitting the pager, while composing the next command.
-
-    .. _less: http://man7.org/linux/man-pages/man1/less.1.html
+    This function was originally inspired by the implementation of
+    `django.core.management.color.supports_color()
+    <https://github.com/django/django/blob/master/django/core/management/color.py>`_
+    but has since evolved significantly.
     """
-    # Compose the pager command.
-    if text and ANSI_CSI in text:
-        command_line = ['less', '--RAW-CONTROL-CHARS']
-    else:
-        command_line = [os.environ.get('PAGER', 'less')]
-    # Pass some additional options to `less' (to make it more
-    # user friendly) without breaking support for other pagers.
-    if os.path.basename(command_line[0]) == 'less':
-        command_line.append('--no-init')
-        command_line.append('--quit-if-one-screen')
-    return command_line
+    if on_windows():
+        # On Windows support for ANSI escape sequences is not a given.
+        have_ansicon = 'ANSICON' in os.environ
+        have_colorama = 'colorama' in sys.modules
+        have_native_support = have_windows_native_ansi_support()
+        if not (have_ansicon or have_colorama or have_native_support):
+            return False
+    return connected_to_terminal(stream)
+
+
+def usage(usage_text):
+    """
+    Print a human friendly usage message to the terminal.
+
+    :param text: The usage message to print (a string).
+
+    This function does two things:
+
+    1. If :data:`sys.stdout` is connected to a terminal (see
+       :func:`connected_to_terminal()`) then the usage message is formatted
+       using :func:`.format_usage()`.
+    2. The usage message is shown using a pager (see :func:`show_pager()`).
+    """
+    if terminal_supports_colors(sys.stdout):
+        usage_text = format_usage(usage_text)
+    show_pager(usage_text)
+
+
+def warning(text, *args, **kw):
+    """
+    Show a warning message on the terminal.
+
+    For details about argument handling please refer to
+    :func:`~humanfriendly.text.format()`.
+
+    Renders the message using :func:`~humanfriendly.text.format()` and writes
+    the resulting string (followed by a newline) to :data:`sys.stderr` using
+    :func:`auto_encode()`.
+
+    If :data:`sys.stderr` is connected to a terminal that supports colors,
+    :func:`ansi_wrap()` is used to color the message in a red font (to make
+    the warning stand out from surrounding text).
+    """
+    text = coerce_string(text)
+    if terminal_supports_colors(sys.stderr):
+        text = ansi_wrap(text, color='red')
+    auto_encode(sys.stderr, text + '\n', *args, **kw)
 
 
 class HTMLConverter(HTMLParser):
@@ -719,6 +855,14 @@ class HTMLConverter(HTMLParser):
       a performance standpoint to re-use an existing :class:`HTMLConverter`
       object for unrelated HTML fragments, in this case take a look at the
       :func:`__call__()` method (it makes this use case very easy).
+
+    .. versionadded:: 4.15
+       :class:`humanfriendly.terminal.HTMLConverter` was added to the
+       `humanfriendly` package during the initial development of my new
+       `chat-archive <https://chat-archive.readthedocs.io/>`_ project, whose
+       command line interface makes for a great demonstration of the
+       flexibility that this feature provides (hint: check out how the search
+       keyword highlighting combines with the regular highlighting).
     """
 
     BLOCK_TAGS = ('div', 'p', 'pre')
@@ -730,13 +874,12 @@ class HTMLConverter(HTMLParser):
 
         :param callback: Optional keyword argument to specify a function that
                          will be called to process text fragments before they
-                         are emitted on the output stream.
-
-                         Note that link text and preformatted text fragments
-                         are not processed by this callback.
+                         are emitted on the output stream. Note that link text
+                         and preformatted text fragments are not processed by
+                         this callback.
         :param output: Optional keyword argument to redirect the output to the
                        given file-like object. If this is not given a new
-                       :class:`python3:~io.StringIO` object is created.
+                       :class:`~python3:io.StringIO` object is created.
         """
         # Hide our optional keyword arguments from the superclass.
         self.callback = kw.pop("callback", None)
