@@ -1,7 +1,7 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: July 16, 2017
+# Last Change: March 1, 2020
 # URL: https://humanfriendly.readthedocs.io
 
 """
@@ -32,10 +32,11 @@ import shutil
 import sys
 import tempfile
 import time
+import unittest
 
 # Modules included in our package.
-from humanfriendly.compat import StringIO, unicode, unittest
-from humanfriendly.text import compact, random_string
+from humanfriendly.compat import StringIO, unicode
+from humanfriendly.text import random_string
 
 # Initialize a logger for this module.
 logger = logging.getLogger(__name__)
@@ -66,8 +67,8 @@ def configure_logging(log_level=logging.DEBUG):
     """configure_logging(log_level=logging.DEBUG)
     Automatically configure logging to the terminal.
 
-    :param log_level: The log verbosity (a number, defaults to
-                      :data:`logging.DEBUG`).
+    :param log_level: The log verbosity (a number, defaults
+                      to :mod:`logging.DEBUG <logging>`).
 
     When :mod:`coloredlogs` is installed :func:`coloredlogs.install()` will be
     used to configure logging to the terminal. When this fails with an
@@ -206,7 +207,7 @@ def run_cli(entry_point, *arguments, **options):
 
 def touch(filename):
     """
-    The equivalent of the UNIX ``touch`` program in Python.
+    The equivalent of the UNIX :man:`touch` program in Python.
 
     :param filename: The pathname of the file to touch (a string).
 
@@ -278,7 +279,7 @@ class PatchedAttribute(ContextManager):
 
 class PatchedItem(ContextManager):
 
-    """Context manager that temporary replaces an object item using :func:`~object.__setitem__()`."""
+    """Context manager that temporary replaces an object item using :meth:`~object.__setitem__()`."""
 
     def __init__(self, obj, item, value):
         """
@@ -464,17 +465,21 @@ class MockedProgram(CustomSearchPath):
     This class extends the functionality of :class:`CustomSearchPath`.
     """
 
-    def __init__(self, name, returncode=0):
+    def __init__(self, name, returncode=0, script=None):
         """
         Initialize a :class:`MockedProgram` object.
 
         :param name: The name of the program (a string).
         :param returncode: The return code that the program should emit (a
                            number, defaults to zero).
+        :param script: Shell script code to include in the mocked program (a
+                       string or :data:`None`). This can be used to mock a
+                       program that is expected to generate specific output.
         """
         # Initialize our own instance variables.
         self.program_name = name
         self.program_returncode = returncode
+        self.program_script = script
         self.program_signal_file = None
         # Initialize our superclasses.
         super(MockedProgram, self).__init__()
@@ -492,6 +497,8 @@ class MockedProgram(CustomSearchPath):
         with open(pathname, 'w') as handle:
             handle.write('#!/bin/sh\n')
             handle.write('echo > %s\n' % pipes.quote(self.program_signal_file))
+            if self.program_script:
+                handle.write('%s\n' % self.program_script.strip())
             handle.write('exit %i\n' % self.program_returncode)
         os.chmod(pathname, 0o755)
         return directory
@@ -580,9 +587,9 @@ class TestCase(unittest.TestCase):
 
     def assertRaises(self, exception, callable, *args, **kwds):
         """
-        Replacement for :func:`unittest.TestCase.assertRaises()` that returns the exception.
+        Replacement for :meth:`unittest.TestCase.assertRaises()` that returns the exception.
 
-        Refer to the :func:`unittest.TestCase.assertRaises()` documentation for
+        Refer to the :meth:`unittest.TestCase.assertRaises()` documentation for
         details on argument handling. The return value is the caught exception.
 
         .. warning:: This method does not support use as a context manager.
@@ -635,25 +642,6 @@ class TestCase(unittest.TestCase):
         """
         return isinstance(exception, tuple(self.exceptionsToSkip))
 
-    def skipTest(self, text, *args, **kw):
-        """
-        Enable skipping of tests.
-
-        This method was added in humanfriendly 3.3 as a fall back for the
-        :func:`unittest.TestCase.skipTest()` method that was added in Python
-        2.7 and 3.1 (because humanfriendly also supports Python 2.6).
-
-        Since then `humanfriendly` has gained a conditional dependency on
-        unittest2_ which enables actual skipping of tests (instead of just
-        mocking it) on Python 2.6.
-
-        This method now remains for backwards compatibility (and just because
-        it's a nice shortcut).
-
-        .. _unittest2: https://pypi.python.org/pypi/unittest2
-        """
-        raise unittest.SkipTest(compact(text, *args, **kw))
-
     def skipTestWrapper(self, test_method, *args, **kw):
         """
         Wrap test methods to translate exceptions into skipped tests.
